import { ChestStatusEnum } from 'models/ActivityChest'
import { NFTHolderRecord } from 'models/NFTHodlerRecord'
import { queryNftBalance } from 'services/chain.svr'
import { generateChestLevel, generateNewChest } from 'services/game.svr'
import { checkDiscordRole } from 'services/oauth.svr'
import { SyncLocker, BaseController, router, role, ROLE_ANON, ZError } from 'zutils'

const nftList = require('../../configs/partner_nft_list.json')
const nftListStr = JSON.stringify(nftList)
const nftMap = new Map()
nftList.forEach(o => nftMap.set(o.contract.toLowerCase(), o))
/**
 * 合作伙伴相关接口
 */
class NftController extends BaseController {
  /**
   * NFT 列表
   */
  @role(ROLE_ANON)
  @router('get /api/partner/nfts')
  async nftList(req) {
    const user = req.user
    let list = JSON.parse(nftListStr)
    list.forEach(o => {
      o.status = 0
    })
    if (user) {
      let records = await NFTHolderRecord.find({ user: user.id })
      let recordSet = new Set(records.map(o => o.contract.toLowerCase()))
      for (let sub of list) {
        list.status = recordSet.has(sub.contract.toLowerCase())
      }
    }
    return list
  }
  /**
   * 领取合作伙伴nft holder奖励, 检查NFT
   */
  // @router('post /api/partner/claim')
  async claimNftHolderReward(req) {
    new SyncLocker().checkLock(req)
    const user = req.user
    let { contract } = req.params
    if (!contract) {
      throw new ZError(11, 'contract not found')
    }
    contract = contract.toLowerCase()
    if (!nftMap.has(contract)) {
      throw new ZError(12, 'contract not found')
    }
    let record = await NFTHolderRecord.findOne({ user: user.id, contract })
    if (record) {
      throw new ZError(13, 'already claimed')
    }
    let rpcRes = await queryNftBalance(contract, user.address)
    console.log('check result:', rpcRes)
    if (rpcRes.error) {
      throw new ZError(20, `check error: ${rpcRes.error.message}`)
    }
    let count = parseInt(rpcRes.result)
    if (count === 0) {
      throw new ZError(14, 'not match claim condition')
    }
    let randomLevel = generateChestLevel()
    let chest = generateNewChest(user.id, user.activity, randomLevel, ChestStatusEnum.NORMAL)
    await chest.save()
    let recordNew = new NFTHolderRecord({
      user: user.id,
      contract,
      chain: nftMap.get(contract).chain,
      holderNum: count,
      rewards: [chest.id],
    })
    await recordNew.save()
    return { chests: [chest.toJson()] }
  }

  @router('post /api/partner/claim')
  async claimNftHolderRewardDC(req) {
    new SyncLocker().checkLock(req)
    const user = req.user
    let { contract } = req.params
    if (!contract) {
      throw new ZError(11, 'contract not found')
    }
    contract = contract.toLowerCase()
    if (!nftMap.has(contract)) {
      throw new ZError(12, 'contract not found')
    }
    let record = await NFTHolderRecord.findOne({ user: user.id, contract })
    if (record) {
      throw new ZError(13, 'already claimed')
    }
    const cfg = nftMap.get(contract)
    let rpcRes = await checkDiscordRole(user.address.toLowerCase(), cfg.guild, cfg.role)
    console.log('check result:', rpcRes)
    if (rpcRes.errcode) {
      throw new ZError(20, `check error: ${rpcRes.errmsg}`)
    }
    if (!rpcRes.data.result) {
      throw new ZError(14, 'not match claim condition')
    }
    let randomLevel = generateChestLevel()
    let chest = generateNewChest(user.id, user.activity, randomLevel, ChestStatusEnum.NORMAL)
    await chest.save()
    let recordNew = new NFTHolderRecord({
      user: user.id,
      contract,
      chain: nftMap.get(contract).chain,
      holderNum: 1,
      rewards: [chest.id],
    })
    await recordNew.save()
    return { chests: [chest.toJson()] }
  }
}
