const DISCORD_API_HOST = 'https://discord.com/api/v10'
/**
 * 直接通过discord接口, 获取工会中某用户信息
 * 返回的数据结构:
 * https://discord.com/developers/docs/resources/guild#guild-member-object
 * //TODO: 须在.env中配置以下参数:
 * DISCROD_GUILD_ID
 * DISCORD_BOT_TOKEN
 */
export async function getGuildMember(guid: string, uid: string) {
  const url = `${DISCORD_API_HOST}/guilds/${guid}/members/${uid}`
  const token = process.env.DISCORD_BOT_TOKEN
  const response = await fetch(url, {
    headers: {
      authorization: `Bot ${token}`,
    },
  })
  const data = await response.json()
  return data
}

/**
 * 检查用户是否已经加入工会
 * @param uid
 * @returns
 */
export async function checkJoinGuld(guid: string, uid: string) {
  try {
    const res = await getGuildMember(guid, uid)
    /**
     * if user not in guild, res will be:
     * {
     *  "message": "Unknown User",
     *  "code": 10013
     *  }
     */
    if (res.code === 10013) {
      return false
    }
    if (res.code) {
      throw new Error(res.message)
    }
    return true
  } catch (e) {
    return false
  }
}
/**
 * 检查用户是否已经获得某个角色
 * @param uid
 * @param roleId
 * @returns
 */
export async function checkGotRole(guid: string, uid: string, roleId: string) {
  try {
    const res = await getGuildMember(guid, uid)
    if (res.code === 10013) {
      return false
    }
    if (res.code) {
      // 如果有其他错误, 直接抛出异常, 让客户端重试
      throw new Error(res.message)
    }
    if (res.roles.includes(roleId)) {
      return true
    }
    return false
  } catch (e) {
    return false
  }
}
