import { hmacSha256 } from 'zutils/utils/security.util'
import { handleFetch, timeoutFetch } from 'zutils/utils/net.util'

const SECRET_KEY = process.env.HASH_SALT
const DEFAULT_TIMEOUT = 30000

function createSign(address: string) {
  address = address.toLowerCase()
  let signCheck = hmacSha256(`address=${address}`, SECRET_KEY)
  return signCheck
}

export async function checkTwitter(address: string) {
  let sign = createSign(address)
  const url = `${process.env.OAUTH_SVR_URL}/activity/twitter/${address}?sign=${sign}`
  return handleFetch(url)
}

export async function checkDiscord(address: string) {
  let sign = createSign(address)
  const url = `${process.env.OAUTH_SVR_URL}/activity/discord/${address}?sign=${sign}`
  return handleFetch(url)
}

export async function checkDiscordGuid(address: string, gid: string) {
  let sign = createSign(address)
  const data = {
    gid,
    address,
    sign,
  }
  const url = `${process.env.OAUTH_SVR_URL}/activity/discord/guild`
  const options: any = {
    method: 'POST',
    headers: {
      'Content-Type': 'application/json; charset=utf-8',
    },
    body: JSON.stringify(data),
  }
  return timeoutFetch(url, options, DEFAULT_TIMEOUT).then(res => res.json())
}

export async function checkDiscordRole(address: string, gid: string, rid: string) {
  let sign = createSign(address)
  const data = {
    gid,
    address,
    rid,
    sign,
  }
  const url = `${process.env.OAUTH_SVR_URL}/activity/discord/role`
  const options: any = {
    method: 'POST',
    headers: {
      'Content-Type': 'application/json; charset=utf-8',
    },
    body: JSON.stringify(data),
  }
  return timeoutFetch(url, options, DEFAULT_TIMEOUT).then(res => res.json())
}
